runUSeq<-function(bam.files, grouping, fdr.thresholds, dir=".", tmpdir=tempfile(tmpdir=getwd()), upath="USeq_8.5.6", 
		rpath="/usr/bin/R", genome="M_musculus_Dec_2011", window=200, fraglen=100, mode="all", dedup=FALSE) 
# Run the USeq pipeline on a set of BAM files, given their differences, and return a list of dataframes
# stating the locations of the called peaks for each FDR threshold. This follows the steps given in 
# http://useq.sourceforge.net/cmdLnMenus.html#ChIPSeq without duplicate read removal (assumed to be
# marked in BAM file and will be removed prior to processing if necessary), read coverage track
# production or peak shifting (using the true values for the fragment lengths).
#
# written by Aaron Lun
{
	stopifnot(file.exists(upath))

	# Setting up the grouping.
	grouping<-factor(grouping)
	stopifnot(length(levels(grouping))==2);

	# Setting up the PointData files (through BED)
	if (file.exists(tmpdir)) { unlink(tmpdir, recursive=TRUE); }
	dir.create(tmpdir);
	pointed<-NULL
	for (i in 1:length(bam.files)) {
	    b.out<-sub("\\.bam$", "", basename(bam.files[i]));
        bed <- file.path(tmpdir, paste0(b.out, ".bed"))
		convertBamToBed(bam.files[i], bed, mode = mode, dedup = dedup)
		cmd<-c("java -Xmx1500M -jar", file.path(upath, "Apps/Tag2Point"), "-f", bed, "-v", genome)
		if (system(paste(cmd, collapse=" "))) { stop("conversion to PointData didn't work") }
		unlink(bed)
		point.out <- file.path(tmpdir, paste0(b.out, "_Point"))
		if (!file.exists(point.out)) { stop("failed to generate PointData from Tag2Point") }
		pointed<-append(pointed, point.out)
	}

	# Running the MultipleScanSeqs command. Using fragment length for the peak shift (reads shifted 
	# by half of this value.).
	treated<-grouping==levels(grouping)[1];
	pdata.treat<-paste(pointed[!treated], collapse=",")
	pdata.control<-paste(pointed[treated], collapse=",")
	cmd<-c("java -Xmx2G -jar", file.path(upath, "Apps/MultipleReplicaScanSeqs"), "-t", pdata.treat,
  		"-c", pdata.control, "-s", 	tmpdir, "-r", rpath, "-w", window, "-p", fraglen)
	if (system(paste(cmd, collapse=" "))) { stop("MultipleReplicaScanSeqs failed") }
	swifname<-file.path(tmpdir, "binaryWindowData.swi.zip")
	if (!file.exists(swifname)) { stop("failed to produce a binary window object") }

	# Running EnrichedRegionMaker to obtain values at each FDR threshold without log-FC filtering.
	all.files<-list()
	for (cutoff in fdr.thresholds) {
        cmd<-c("java -Xmx2G -jar", file.path(upath, "Apps/EnrichedRegionMaker"), "-f",
           swifname, "-i 0,1 -s", paste(-10*log10(cutoff), 0, sep=","), "-t", pdata.treat, "-c", pdata.control, "-p", rpath);
        if (system(paste(cmd, collapse=" "))) { stop("EnrichedRegionMaker failed for forward enrichment") }
        if (system(paste(c(cmd, "-m"), collapse=" "))) { stop("EnrichedRegionMaker failed for reverse enrichment") }
    	out<-list.files(tmpdir, pattern="Regions_BinaryWindowData");

		ofname<-file.path(dir, paste0("regions-", cutoff, ".tsv"))
		all.files[[as.character(cutoff)]]<-ofname
    	outhandle<-file(ofname, open="w")
    	write(file=outhandle, c("chr", "start", "end", "PValue", "FDR", "logFC", "logCPM"), ncolumns=7, sep="\t")

    	for (out.x in out) {
        	tango<-list.files(file.path(tmpdir, out.x), pattern='xls$');
        	if (length(tango)>1) { 
				warning("multiple XLS files, choosing just 1") 
				tango<-tango[1]
			} else if (!length(tango)) { next  }

           	d<-read.table(file.path(tmpdir, out.x, tango), comment.char="", header=TRUE, fill=TRUE);
            if (nrow(d)) {
				# Checking arguments before filling in everything.
				start.col <- grep("[ER]R_Start", colnames(d))
				end.col <- grep("[ER]R_Stop", colnames(d))
				if (any(is.na(match(c("Chr", "BSW_BinPVal", "BW_FDR"), colnames(d)))) ||
						length(end.col)!=1L || length(start.col)!=1L) { 
					stop("absent columns in the Useq output file") }
                write.table(file=outhandle, x=data.frame(d$Chr, d[,start.col], d[,end.col], 
					10^-(d$BSW_BinPVal/10), 10^-(d$BW_FDR/10), d[,18], 0), 
					sep="\t", quote=FALSE, row.names=FALSE, col.names=FALSE);
            }
    	}
    	close(outhandle)

		# Need to kill all these directories, to avoid them being picked up at
		# the next cut-off.
    	unlink(file.path(tmpdir, out), recursive=TRUE);
	}

	# Erasing the output directory to clean up.
	unlink(tmpdir, recursive=TRUE)
	return(all.files);
}


