spawnTADs <- function(ntads, min.tad=10, max.tad=30) 
# This function spawns counts for TADs. It returns a list of matrices
# indicating the pairs of restriction site indices in each TAD.
{
	ntads <- as.integer(ntads)
	mysize <- as.integer(runif(ntads, min.tad, max.tad))
	tads <- list()
	last.launch <- 0L
	for (i in 1:ntads) { 
		current <- last.launch + mysize[i]:1
		tads[[i]] <- t(combn(current, 2L))
		colnames(tads[[i]]) <- c("anchor", "target")
		last.launch <- last.launch + mysize[i]
	}
	tads
}

spawnDiagon <- function(len) 
# Adds coordinates for the diagonals. 
{
	len <- as.integer(len)
	cbind(anchor=1:len, target=1:len)
}

spawnLigations <- function(nspec, max.range, len, diag=TRUE) 
# This adds ligation events within a certain range of each other. This avoids
# simulating the far-out interactions which are mostly irrelevant.
{ 
	max.range <- as.integer(max.range)
	len <- as.integer(len)
	if (max.range > len) { stop("max.range must be less than the total length of the chromosome") }

	per.height <- pmin(max.range + 1L, len-1:len+1L)
	if (!diag) { per.height <- per.height[-len] - 1L }
	total.possibles <- sum(per.height)
	if (total.possibles <= nspec) { 
		warning("reporting everything as requested") 
		chosen <- 1:total.possibles
	} else {
		chosen <- sample(total.possibles, nspec) 
	}
	
	ratchet <- c(1L, cumsum(per.height)+1L)
	target <- findInterval(chosen, ratchet) 
	anchor <- chosen - ratchet[target] + target
	if (!diag) { anchor <- anchor + 1L }
	cbind(anchor=anchor, target=target)
}

spawnLigations2 <- function(nspec, len, diag=FALSE, prior=1, decay=0.5) 
# This adds ligation events where the weighting of the selected ligation events is
# skewed towards the diagonal. This avoids putting too much work into the rest of the
# contact matrix that is unlikely to be interesting.
{ 
	len <- as.integer(len)
	all.dists <- (len-1):ifelse(diag, 0, 1)
	n.entries <- len - all.dists
	chosen.dists <- sample(all.dists, nspec, p=n.entries*(all.dists+prior)^(-decay), replace=TRUE) 

	chosen <- split(1:nspec, chosen.dists)
	anchor <- target <- integer(nspec)
	for (dist in names(chosen)) {
		relevants <- chosen[[dist]]
		allowable.anchors <- (as.integer(dist)+1L):len
		cur.anchors <- sample(allowable.anchors, length(relevants), replace=TRUE)
		cur.targets <- cur.anchors - as.integer(dist) 
		anchor[relevants] <- cur.anchors
		target[relevants] <- cur.targets
	}

	o <- order(anchor, target)
	cbind(anchor=anchor[o], target=target[o])
}

expandLigations <- function(mat, width, len) 
# This expands each ligation event to account for increased width of the 
# interaction. It also stores the output in a list to indicate what
# interaction corresponds to whom.
{
	storage.mode(mat) <- "integer"
	width <- as.integer(width)
	len <- as.integer(len)
	out <- .Call("expand_coords", mat, nrow(mat), width, len, PACKAGE="xscss")
	if (is.character(out)) { stop(out) }
	out
}

getMeans <- function(mat, base=10, decay=0.5, prior=1, rate=1) 
# This makes mean values for the requested matrices that are coming in,
# based on the log-log relationship observed between mean and distance.
{
	distance <- mat[,1]-mat[,2] + prior
	true.mean <- base*distance^(-decay)
	if (is.na(rate)) { return(true.mean) } 
	return(rgamma(nrow(mat), shape=true.mean*rate, rate=rate))
}

aggregateAll <- function(...) 
# This takes a bunch of (coordinate matrix, count vector) lists, and
# merges them to compute the aggregate mean.
{
	everything <- list(...)
	all.anchors <- all.targets <- all.counts <- list()
	counter <- 1L
	for (x in everything) { 
		all.anchors[[counter]] <- x[[1]][,1]
		all.targets[[counter]] <- x[[1]][,2]
		all.counts[[counter]] <- x[[2]]
		counter <- counter + 1L
	}

	# Reordering.
	all.anchors <- unlist(all.anchors)
	all.targets <- unlist(all.targets)
	all.counts <- unlist(all.counts)
	o <- order(all.anchors, all.targets)
	all.anchors <- all.anchors[o]
	all.targets <- all.targets[o]
	all.counts <- all.counts[o]
	
	# Finding the sum.
	keep <- which(diff(all.anchors)!=0L | diff(all.targets)!=0L)
	first <- c(1L, keep + 1L)
	last <- c(keep, length(o))
	cum.count <- cumsum(all.counts)
	collected <- cum.count[last] - cum.count[first] + all.counts[first]

	# Returning the aggregated output.
	list(cbind(anchor=all.anchors[first], target=all.targets[first]), collected)
}


dummyFragments <- function(len, name="chrA") {
	names(len) <- name
	GRanges(name, IRanges(1:len, 1:len), seqlengths=len)
}


