#include <cstdio>
#include <stdexcept>
#include <iostream>

#include "R.h"
#include "Rdefines.h"
#include "Rmath.h"

void fill_strings(const int& r, char*& mockseq, char*& mockqual) {
	mockseq=(char*)R_alloc(r+1, sizeof(char));
	mockqual=(char*)R_alloc(r+1, sizeof(char));
	for (int i=0; i<r; ++i) { 
		mockseq[i]='N'; 
		mockqual[i]='h';
	}
	mockseq[r]='\0';
	mockqual[r]='\0';
	return;
}

void precheck(SEXP fname, SEXP chr, SEXP width, SEXP y, SEXP rlen) {
    if (!IS_CHARACTER(fname) || LENGTH(fname)!=1) { throw std::runtime_error("file name should be a character string"); }
	if (!IS_CHARACTER(chr) || LENGTH(chr)!=1 ) { throw std::runtime_error("chr name should be a character string"); }
	if (!IS_INTEGER(width) || LENGTH(width)!=1) { throw std::runtime_error("width should be an integer scalar"); }
	if (!IS_INTEGER(y)) { throw std::runtime_error("number of reads per bin should be an integer vector"); }
	if (!IS_INTEGER(rlen) || LENGTH(rlen)!=1) { throw std::runtime_error("read length should be an integer scalar"); }
}

extern "C" {

SEXP R_dump_background (SEXP fname, SEXP chr, SEXP width, SEXP y, SEXP rlen) try {
	precheck(fname, chr, width, y, rlen);

	const char* chrname=CHAR(STRING_ELT(chr, 0));
    const int n=LENGTH(y);
	const int* yptr=INTEGER_POINTER(y);
	const int w=INTEGER_VALUE(width), r=INTEGER_VALUE(rlen);
	char*mockseq=0, *mockqual=0;
	fill_strings(r, mockseq, mockqual);
        
    // Iterating through the blocks.    
    GetRNGstate();
	FILE* ohandle=fopen(CHAR(STRING_ELT(fname, 0)), "a");
	if (ohandle==NULL) { throw std::runtime_error("could not open output file"); }
   
	int position=1;
    for (int i=0; i<n; ++i) {
		const int& cury=yptr[i];
		for (int j=0; j<cury; ++j) {
            const bool forward=(unif_rand()>0.5);
			int cur_pos=position+int(unif_rand()*w);
			if (fprintf(ohandle, "r.%s.%i.%i\t%i\t%s\t%i\t200\t%iM\t*\t0\t0\t%s\t%s\n", chrname, j, i, 
					(forward ? 0 : 16), chrname, cur_pos, r, mockseq, mockqual) <0) {
				throw std::runtime_error("write error"); 
			}
		}	
		position+=w;

	}
	fclose(ohandle);
    PutRNGstate();
    return ScalarLogical((Rboolean) TRUE);
} catch (std::exception& e) {
	return mkString(e.what());	
}

SEXP R_dump_tf (SEXP fname, SEXP chr, SEXP pos, SEXP y, SEXP fraglen, SEXP rlen, SEXP max) try {
	precheck(fname, chr, fraglen, y, rlen);
	if (!IS_INTEGER(pos)) { throw std::runtime_error("position should be an integer vector"); }
	if (!IS_INTEGER(max) || LENGTH(max)!=1) { throw std::runtime_error("length of chr should be an integer scalar"); }

	const char* chrname=CHAR(STRING_ELT(chr, 0));
    const int n=LENGTH(y);
	const int* yptr=INTEGER_POINTER(y);
	const int* pptr=INTEGER_POINTER(pos);
	if (LENGTH(pos)!=n) { throw std::runtime_error("length of spike-in vectors are not equal"); }
	const int f=INTEGER_VALUE(fraglen), r=INTEGER_VALUE(rlen), m=INTEGER_VALUE(max);
  	char*mockseq=0, *mockqual=0;
	fill_strings(r, mockseq, mockqual);
 
    // Iterating through the blocks.    
    GetRNGstate();
	FILE* ohandle=fopen(CHAR(STRING_ELT(fname, 0)), "a");
	if (ohandle==NULL) { 
		throw std::runtime_error("could not open output file");
	}
   
    for (int i=0; i<n; ++i) {
		const int& cury=yptr[i];
		const int& curp=pptr[i];
		for (int j=0; j<cury; ++j) {
            const bool forward=(unif_rand()>0.5);
			int cur_pos=curp+(forward ? -1 : 1)*rbeta(2, 2)*f;
			if (!forward) { cur_pos-=r-1; } // Need to record the 5'-most end, not the 5' end of the read.
			if (cur_pos > m || cur_pos < 1) { continue; }
			if (fprintf(ohandle, "r.%s.%i.%i\t%i\t%s\t%i\t200\t%iM\t*\t0\t0\t%s\t%s\n", chrname, j, i, 
					(forward ? 0 : 16), chrname, cur_pos, r, mockseq, mockqual) <0) {
				throw std::runtime_error("write error"); 
			}
		}	
	}
	fclose(ohandle);
    PutRNGstate();
    return ScalarLogical((Rboolean) TRUE);
} catch (std::exception& e) {
	return mkString(e.what());	
}

SEXP R_dump_histone (SEXP fname, SEXP chr, SEXP pos, SEXP y, SEXP width, SEXP rlen, SEXP max, SEXP alpha, SEXP beta) try {
	precheck(fname, chr, width, y, rlen);
	if (!IS_INTEGER(pos)) { throw std::runtime_error("position should be an integer vector"); }
	if (!IS_INTEGER(max) || LENGTH(max)!=1) { throw std::runtime_error("length of chr should be an integer scalar"); }
	if (!IS_NUMERIC(alpha) || LENGTH(alpha)!=1 || !IS_NUMERIC(beta) || LENGTH(beta)!=1) { 
		throw std::runtime_error("alpha and beta should be numeric scalars"); 
	}

	const char* chrname=CHAR(STRING_ELT(chr, 0));
    const int n=LENGTH(y);
	const int* yptr=INTEGER_POINTER(y);
	const int* pptr=INTEGER_POINTER(pos);
	if (LENGTH(pos)!=n) { throw std::runtime_error("length of spike-in vectors are not equal"); }
	const int w=INTEGER_VALUE(width), r=INTEGER_VALUE(rlen), m=INTEGER_VALUE(max);
	const double alp=NUMERIC_VALUE(alpha), bet=NUMERIC_VALUE(beta);
  	char*mockseq=0, *mockqual=0;
	fill_strings(r, mockseq, mockqual);
 
    // Iterating through the blocks.    
    GetRNGstate();
	FILE* ohandle=fopen(CHAR(STRING_ELT(fname, 0)), "a");
	if (ohandle==NULL) { 
		throw std::runtime_error("could not open output file");
	}
   
    for (int i=0; i<n; ++i) {
		const int& cury=yptr[i];
		const int& curp=pptr[i];
		for (int j=0; j<cury; ++j) {
            const bool forward=(unif_rand()>0.5);
			int cur_pos=curp-w/2+int(w*rbeta(alp, bet));
            if (cur_pos > m || cur_pos < 1) { continue; }
			if (fprintf(ohandle, "r.%s.%i.%i\t%i\t%s\t%i\t200\t%iM\t*\t0\t0\t%s\t%s\n", chrname, j, i, 
					(forward ? 0 : 16), chrname, cur_pos, r, mockseq, mockqual) <0) {
				throw std::runtime_error("write error"); 
			}
		}	
	}
	fclose(ohandle);
    PutRNGstate();
    return ScalarLogical((Rboolean) TRUE);
} catch (std::exception& e) {
	return mkString(e.what());	
}
}
