####################################################################################################
# This script tests the BAM --> BED conversion procedure, by checking the commands for all options.

suppressPackageStartupMessages(require(xscss))
suppressPackageStartupMessages(require(Rsamtools))

convertBamToBed("blah.bam", "stuff.bed", mode="all", dedup=FALSE, cmd.only=TRUE)

convertBamToBed("blah.bam", "stuff.bed", mode="all", dedup=TRUE, cmd.only=TRUE)

convertBamToBed("blah.bam", "stuff.bed", mode="all", dedup=TRUE, minq=100, cmd.only=TRUE)

convertBamToBed("blah.bam", "stuff.bed", mode="nuclear", dedup=FALSE, cmd.only=TRUE)

convertBamToBed("blah.bam", "stuff.bed", mode="nuclear", dedup=FALSE, minq=100, cmd.only=TRUE)

convertBamToBed("blah.bam", "stuff.bed", mode="autosomal", dedup=FALSE, cmd.only=TRUE)

convertBamToBed(c("blah.bam", "blax.bam"), "stuff.bed", mode="autosomal", dedup=FALSE, cmd.only=TRUE)

try(convertBamToBed(c("blah.bam", "blax.bam"), c("whee.bed", "stuff.bed"), mode="autosomal", dedup=FALSE, cmd.only=TRUE))

####################################################################################################
# We also simulate a couple of files and we throw them in to see how it performs.

odir<-"temp"
dir.create(odir)
fname<-file.path(odir, "out.sam")

autocomp <- function() {
	n<-10
	sizes<-rep(10000, 6)
	names(sizes)<-c("chr1", "chr21", "chrM", "chrX", "chrY", "chr12_random")
	chrs<-sample(names(sizes), n, replace=T)
    mu<-rep(20, n)
	pos<-round(runif(n, 2000, 8000))
	disp<-rep(0.05, n)
	peakFile(fname, chrs=chrs, pos=pos, mu=mu, disp=disp, sizes=sizes)

	# Running through it and adding random odds and ends.
	blah<-read.table(fname, header=FALSE, comment="@")
	chosen<-rbinom(nrow(blah), 1, 0.1)==1
	blah[chosen,2]<-blah[chosen,2]+1024
	altname<-file.path(odir, "temp.sam")
	if (system(paste("cat", fname, "| grep '^@' >", altname))) { stop("whee") }
	write.table(file=altname, blah, col.names=FALSE, row.names=FALSE, sep="\t", quote=FALSE,  append=TRUE)
	file.rename(altname, fname)

	# Setting up the table of known results.
	rnames<-as.character(blah[,1])
	true.start<-blah[,4]-1L
	true.chr<-as.character(blah[,3])
	true.end<-true.start+nchar(as.character(blah[,10]))

	# Crunching it and converting it. Comparing each run to the table of known results.
	bam<-crunch2BAM(fname, dir=odir)
	bname<-file.path(odir, "stuff.bed")

	funcheck<-function(tokeep) {
		if (!file.info(bname)$size) {
			if (sum(tokeep)) { stop("some reads should be present") }
			cat("All reads discarded correctly\n")
			return(invisible(NULL))
		}
		comparator<-read.table(bname,stringsAsFactors=FALSE)
		stopifnot(sum(tokeep)==nrow(comparator))
		o<-match(rnames[tokeep], comparator[,4])
		stopifnot(all(!is.na(o)))
		stopifnot(identical(true.chr[tokeep], comparator[o,1]))
		stopifnot(identical(true.start[tokeep], comparator[o,2]))
		stopifnot(identical(true.end[tokeep], comparator[o,3]))
		stopifnot(identical(bitwAnd(blah[tokeep,2], 0x10)==0x10, comparator[o,ncol(comparator)]=="-"))
		cat("Total of", sum(tokeep), "reads\n")
	}

	convertBamToBed(bam, bname, mode="all")
	funcheck(rep(TRUE, nrow(blah)))
	convertBamToBed(bam, bname, mode="all", dedup=TRUE)
	funcheck(!chosen)
	convertBamToBed(bam, bname, mode="all", dedup=TRUE, minq=255)
	funcheck(rep(FALSE, nrow(blah)))

	nuked<-true.chr %in% names(sizes)[c(1,2,4,5)]
	convertBamToBed(bam, bname, mode="nuclear")
	funcheck(nuked)
	convertBamToBed(bam, bname, mode="nuclear", dedup=TRUE)
	funcheck(nuked & !chosen)

	autos<-true.chr %in% names(sizes)[c(1,2)]
	convertBamToBed(bam, bname, mode="autosomal")
	funcheck(autos)
	convertBamToBed(bam, bname, mode="autosomal", dedup=TRUE)
	funcheck(autos & !chosen)
}

set.seed(29873)
autocomp()

autocomp()

autocomp()

unlink(odir, recursive=TRUE)

####################################################################################################
# End.
