####################################################################################################
# This tests the simulation scripts by checking that they indeed generate what we request. We just
# pump out a couple of simulations, reset the seeds, and recompute the counts, and then count
# over the intervals to check that the counts per region are actually what was ordered.

suppressPackageStartupMessages(require(xscss))
set.seed(1348975)
invisible(runif(1e6))
all.seeds <- round(runif(10)*.Machine$integer.max)

tmpdir<-"tmp"
dir.create(tmpdir)
sizes <- c(chrA=3e5, chrB=1e6, chrC=6e6)

####################################################################################################

loadbg <- function(seed.incoming, rlen=10, width=2000L, back.mu = c(10, 50), prior.df = 20, dispersion = 0.05) {
	set.seed(seed.incoming)
	fnames <- file.path(tmpdir, paste0("stuff", 1:2, ".sam"))
	addBackground(fnames,  sizes, rlen=rlen, width=width, back.mu=back.mu, prior.df=prior.df, dispersion=dispersion)

	# Pulling the counts out of the file.
	collected <- list()
	for (x in fnames) {
		everything <- read.table(x, comment.char="@", 
			colClasses=list(NULL, NULL, "character", "integer", 
				NULL, NULL, NULL, NULL, NULL, NULL, NULL))
		y <- split(everything[,2], everything[,1])

		# Pulling them out for each bin.
		curcounts <- list()
		for (chr in names(y)) {
			ix <- as.integer((y[[chr]]-1L)/width)+1L
			curcounts[[chr]] <- tabulate(ix, nbins=as.integer(sizes[[chr]]/width))
		}
		collected[[x]] <- unlist(curcounts, use.names=FALSE)
	}
	collected <- do.call(cbind, collected)
	colnames(collected) <- NULL

	# Trying to simulate said counts. Need to pretend to put in the reads (strand, positional definition).
	set.seed(seed.incoming)
	refcol <- list(NULL, NULL)
	names(refcol) <- fnames

    for (x in names(sizes)) {
		nbins<-as.integer(sizes[[x]]/width)
		bmu<-runif(nbins, back.mu[1], back.mu[2])
		bdisp<-dispersion*prior.df/rchisq(nbins, df=prior.df)
		for (fname in fnames) {
			current <- as.integer(rnbinom(nbins, mu=bmu, size=1/bdisp)+0.5)
			refcol[[fname]] <- append(refcol[[fname]], current)
			invisible(runif(sum(current)*2))
		}
	}
	refcol <- do.call(cbind, refcol)
	colnames(refcol) <- NULL

	if (!identical(refcol, collected)) {
		stop("counts don't match up between simulation and what was requested")
	}
	head(collected)
}

loadbg(all.seeds[1])

loadbg(all.seeds[2], rlen=20)

loadbg(all.seeds[3], rlen=50, back.mu=c(50, 100), width=5000)

loadbg(all.seeds[4], back.mu=c(5, 20), width=1000)

loadbg(all.seeds[5], prior.df=50, width=2500)

loadbg(all.seeds[6], dispersion=0.1)

loadbg(all.seeds[7], width=10000, back.mu=c(100, 200))

loadbg(all.seeds[8], width=2752, back.mu=c(20, 200))

loadbg(all.seeds[9], width=5683, prior.df=50, dispersion=0.01)

loadbg(all.seeds[10], width=1954)

####################################################################################################
# Working with TFs. 

loadtf <- function(seed.incoming, npeaks=100, dist.range=c(500, 1000), 
		mu.range=c(20, 200), fraglen=100, rlen=10, prior.df = 20, dispersion = 0.05) {
	chrs <- rep(names(sizes), rep(npeaks, length(sizes)))
	all.pos <- list()
	for (chr in names(sizes)) {
		pos <- cumsum(round(runif(npeaks, dist.range[1], dist.range[2])))
		all.pos[[chr]]<- pos
	}
	all.pos <- unlist(all.pos)
	disp <- dispersion*prior.df / rchisq(length(chrs), df=prior.df)
	mus <- runif(length(chrs), mu.range[1], mu.range[2])
	
	set.seed(seed.incoming)
	fname <- file.path(tmpdir, "stuff.sam")
	peakFile(fname, chrs=chrs, pos=all.pos, sizes=sizes, rlen=rlen, disp=disp, mus=mus, fraglen=fraglen)

	# Pulling the counts out of the file for each peak. Checking that there isn't anything 
	# on the wrong strand (enforce bimodality).
	collected <- list()
	everything <- read.table(fname, comment.char="@", 
		colClasses=list(NULL, "integer", "character", "integer", 
			NULL, NULL, NULL, NULL, NULL, NULL, NULL))
	neg.ranges <- GRanges(chrs, IRanges(all.pos, all.pos+fraglen-1))
	pos.ranges <- GRanges(chrs, IRanges(all.pos-fraglen, all.pos-1))
	isforward <- bitwAnd(everything[,1], 0x10)==0L
	all.ranges <- GRanges(everything[,2], IRanges(everything[,3], everything[,3]+rlen-1))
	collected <- countOverlaps(pos.ranges, all.ranges[isforward]) + countOverlaps(neg.ranges, all.ranges[!isforward])
	
	# Simulating said counts. Don't need to do the rest.
	set.seed(seed.incoming)
    y<-as.integer(rnbinom(length(all.pos), mu=mus, size=1/disp)+0.5)

	if (!identical(collected,y)) { stop("counts don't match up between simulation and what was requested") }
	head(collected)
}

set.seed(1348975)
invisible(runif(2e6))
all.seeds <- round(runif(10)*.Machine$integer.max)

loadtf(all.seeds[1])

loadtf(all.seeds[2], rlen=20, dist.range=c(200, 500))

loadtf(all.seeds[3], rlen=50, mu.range=c(50, 100), fraglen=500, dist.range=c(1000, 2000))

loadtf(all.seeds[4], mu.range=c(5, 20), fraglen=50)

loadtf(all.seeds[5], prior.df=50, fraglen=250)

loadtf(all.seeds[6], dispersion=0.1)

loadtf(all.seeds[7], fraglen=1000, mu.range=c(100, 200), dist.range=c(2000, 3000)) # Don't go too high; max chromosome A is 3e5

loadtf(all.seeds[8], fraglen=275, mu.range=c(20, 200), rlen=182)

loadtf(all.seeds[9], fraglen=563, prior.df=50, dispersion=0.01, dist.range=c(1000, 3000))

loadtf(all.seeds[10], fraglen=154, rlen=57)

####################################################################################################
# Working with histones.

loadhs <- function(seed.incoming, npeaks=100, dist.range=c(1500, 2500), 
		mu.range=c(20, 200), width=1000, rlen=10, prior.df = 20, dispersion = 0.05) {
	chrs <- rep(names(sizes), rep(npeaks, length(sizes)))
	all.pos <- list()
	for (chr in names(sizes)) {
		pos <- cumsum(round(runif(npeaks, dist.range[1], dist.range[2])))
		all.pos[[chr]]<- pos
	}
	all.pos <- unlist(all.pos)
	disp <- dispersion*prior.df / rchisq(length(chrs), df=prior.df)
	mus <- runif(length(chrs), mu.range[1], mu.range[2])
	
	set.seed(seed.incoming)
	fname <- file.path(tmpdir, "stuff.sam")
	peakFile(fname, chrs=chrs, pos=all.pos, sizes=sizes, rlen=rlen, disp=disp, mus=mus, width=width, tf=FALSE)

	# Pulling the counts out of the file for each peak. Checking that there isn't anything 
	# on the wrong strand (enforce bimodality).
	collected <- list()
	everything <- read.table(fname, comment.char="@", 
		colClasses=list(NULL, NULL, "character", "integer", 
			NULL, NULL, NULL, NULL, NULL, NULL, NULL))
	tango.ranges <- GRanges(chrs, IRanges(all.pos-width/2-1, all.pos+width/2+1))
	all.ranges <- GRanges(everything[,1], IRanges(everything[,2], everything[,2]))
	collected <- countOverlaps(tango.ranges, all.ranges) 
	
	# Simulating said counts. Don't need to do the rest.
	set.seed(seed.incoming)
    y<-as.integer(rnbinom(length(all.pos), mu=mus, size=1/disp)+0.5)

	if (!identical(collected,y)) { stop("counts don't match up between simulation and what was requested") }
	head(collected)
}

set.seed(1348975)
invisible(runif(3e6))
all.seeds <- round(runif(10)*.Machine$integer.max)

loadhs(all.seeds[1])

loadhs(all.seeds[2], rlen=20, dist.range=c(1100, 1500))

loadhs(all.seeds[3], rlen=50, mu.range=c(50, 100), width=500, dist.range=c(1000, 2000))

loadhs(all.seeds[4], mu.range=c(5, 20), width=50)

loadhs(all.seeds[5], prior.df=50, width=250)

loadhs(all.seeds[6], dispersion=0.1)

loadhs(all.seeds[7], width=1800, mu.range=c(100, 200), dist.range=c(2000, 3000)) 

loadhs(all.seeds[8], width=2705, mu.range=c(20, 200), rlen=182, dist.range=c(2800, 2850))

loadhs(all.seeds[9], width=563, prior.df=50, dispersion=0.01)

loadhs(all.seeds[10], width=1540, rlen=57, dist.range=c(2000, 3000))

####################################################################################################
# Cleaning up.

unlink(tmpdir, recursive=TRUE)

####################################################################################################
####################################################################################################
# End.

